setwd (dir = '/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/focal-terms/analysis')
library(rgl)
library (cluster)

distMat_ac = readRDS('distMat_acoustic_best-weighted.RDS')

distMatS_sv = readRDS ('distMatS_sv.RDS')
distMatS_en = readRDS ('distMatS_en.RDS')
distMatS_ru = readRDS ('distMatS_ru.RDS')

distMatEm_sv = readRDS ('distMatEm_sv.RDS')
distMatEm_en = readRDS ('distMatEm_en.RDS')
distMatEm_ru = readRDS ('distMatEm_ru.RDS')

table_simpl_sv = read.csv ('table_simpl_sv.csv')[,-1]
table_simpl_en = read.csv ('table_simpl_en.csv')[,-1]
table_simpl_ru = read.csv ('table_simpl_ru.csv')[,-1]

# triplets 3D configurations
triplets_en = read.csv ("triplets_map_3D_en.csv")[,-1]
triplets_sv = read.csv ("triplets_map_3D_sv.csv")[,-1]
# triplets distance matrices for Swedish / English participants only  
distMat_triplets_en = as.dist(read.csv ('distMat_triplets_en.csv')[,-1])
distMat_triplets_sv = as.dist(read.csv ('distMat_triplets_sv.csv')[,-1])


## Correlation of distance matrices
mycors = cor ( cbind(distMat_ac,
                     distMatS_en, distMatS_sv, distMatS_ru, 
                     (distMatS_en+distMatS_sv+distMatS_ru)/3,
                     distMatEm_en, distMatEm_sv, distMatEm_ru,
                     (distMatEm_en+distMatEm_sv+distMatEm_ru)/3,
                     distMat_triplets_en, distMat_triplets_sv,
                     (distMat_triplets_en+distMat_triplets_sv)/2) )

write.csv (round(mycors,2), '~/Downloads/cors.csv')


## plotting
lang = c('en','sv')[2] 

if (lang=='sv') {
  dfTemp = table_simpl_sv
  dfTemp$topCall = as.character(dfTemp$topCall)
  dfTemp$topCall = ifelse (dfTemp$topCall=='callDK','DK',dfTemp$topCall)
  dfTemp$topCall = as.factor(dfTemp$topCall)
  calls = levels(dfTemp$topCall)
  calls_en = c ('DK', 'gasp', 'giggle', 'cry', 'yell', 'laugh', 'scream', 'moan', 'sigh', 'howl', 'roar')
  scores = triplets_sv[,1:3]
  dfTemp$call_en = calls_en[match(dfTemp$topCall, calls)]
  calls = sort(calls)
} else if (lang=='en') {
  dfTemp = table_simpl_en
  dfTemp$topCall = as.character(dfTemp$topCall)
  dfTemp$topCall = ifelse (dfTemp$topCall=='callDK','DK',dfTemp$topCall)
  dfTemp$topCall = as.factor(dfTemp$topCall)
  calls = levels(dfTemp$topCall)
  calls = sort(calls)
  calls_en = calls
  scores = triplets_en[,1:3]
  dfTemp$call_en = dfTemp$topCall
} 

# silhouette
sil = silhouette (x=as.numeric(dfTemp$topCall), dist=dist(scores)) 
s = summary(sil)
print (paste('Silhouette:', weighted.mean (s$clus.avg.widths, w=s$clus.sizes)) ) # -0.03 for sv, -0.12 for en

dfTemp$mds1 = scores[,1]
dfTemp$mds2 = scores[,2]
dfTemp$mds3 = scores[,3]

centroids = matrix(ncol=3, nrow=length(levels(dfTemp$topCall)))
for (l in 1:length(levels(dfTemp$topCall))) {
  a = dfTemp[dfTemp$topCall==levels(dfTemp$topCall)[l],]
  weights = a$sureSound/a$callEntropy; weights = weights/sum(weights)
  centroids[l,] = apply (a[,c('mds1','mds2','mds3')], 2, function(x)sum(x*weights))
} # mean weighted by sureSound and /callEntropy - shows centroid taking certainty into account
colnames(centroids) = c('mds1','mds2','mds3')
centroids = as.data.frame(centroids)
centroids$topCall = levels(dfTemp$topCall)


key = data.frame (allCalls = c("cry","DK","gasp","giggle","groan","growl","grunt","howl","laugh","moan","roar","scream","shriek","sigh","sob","yell"), allShapes = c(0:5,8:13,15:18), allColors=c('#d376dc','#903099','#dd345f','#42268d','#5c36ca','#c53fe1','#d43e88','#dd3eb7','#6de56f','#a85ae3','#b2d63f','#dab433','#d86f27','#52a437','#d03f30','#6a6de0'))
myCols = as.character(key$allColors[match(sort(unique(dfTemp$call_en)), key$allCalls)])
palette (myCols)
dfTemp$col_as_number = match(dfTemp$call_en, key$allCalls)
# palette (rainbow(length(levels(dfTemp$topCall))))

# optional: rotate to standardize view
scores[,c(1)] = -scores[,c(1)]  # specify which axes to invert
centroids[,c(1)] = -centroids[,c(1)] # same here


### Adding axes
## First draw a line from center of gravity to gråt / cry
ooo = apply(scores[,1:3], 2, mean)
if (lang=='sv'){
  d1 = as.numeric (centroids[centroids$topCall=='gråt',1:3][1] - ooo[1])
  d2 = as.numeric (centroids[centroids$topCall=='gråt',1:3][2] - ooo[2])
  d3 = as.numeric (centroids[centroids$topCall=='gråt',1:3][3] - ooo[3])
} else if (lang=='en'){
  d1 = as.numeric (centroids[centroids$topCall=='cry',1:3][1] - ooo[1])
  d2 = as.numeric (centroids[centroids$topCall=='cry',1:3][2] - ooo[2])
  d3 = as.numeric (centroids[centroids$topCall=='cry',1:3][3] - ooo[3])
}

v1 = c(d1, d2, d3)

# line1 = ooo + t * v1
point1_1 = ooo - 1 * v1
point1_2 = ooo + 1.2 * v1

## Then we draw a line from ooo towards laugh, but perpendicular to the first line
if (lang=='sv'){
  d2_1 = as.numeric (centroids[centroids$topCall=='skratt',1:3][1] - ooo[1])
  d2_2 = as.numeric (centroids[centroids$topCall=='skratt',1:3][2] - ooo[2])
  d2_3 = as.numeric (centroids[centroids$topCall=='skratt',1:3][3] - ooo[3])
} else if (lang=='en'){
  d2_1 = as.numeric (centroids[centroids$topCall=='laugh',1:3][1] - ooo[1])
  d2_2 = as.numeric (centroids[centroids$topCall=='laugh',1:3][2] - ooo[2])
  d2_3 = as.numeric (centroids[centroids$topCall=='laugh',1:3][3] - ooo[3])
}

v2 = c (d2_1, d2_2, d2_3)

perp = function (v1, v2) {
  return (abs(v1 %*% v2))
}
perp (v1, v2)

nIter = 1000
for (i in 1:nIter) {   # I suppose it can be done analytically, but here 
  displ = rnorm (3, 0, 0.1) # we just wriggle the line around laugh until it's ~90°
  vTemp = v2+displ
  if (perp(v1,vTemp) < perp(v1,v2)) {
    v2 = vTemp
  }
}
acos( sum(v1*v2) / ( sqrt(sum(v1 * v1)) * sqrt(sum(v2 * v2)) ) ) * 180 / pi  # angle

# line2 = ooo + t * v2
point2_1 = ooo - v2
point2_2 = ooo + 1.2 * v2

## now we find a third vector, perpendicular to both v1 and v2
# http://www.wikihow.com/Find-Perpendicular-Vectors-in-3-Dimensions
temp1 = det ( matrix(ncol=2, c(v1[c(2,3)],v2[c(2,3)])) )
temp2 = -det ( matrix(ncol=2, c(v1[c(1,3)],v2[c(1,3)])) )
temp3 = det ( matrix(ncol=2, c(v1[c(1,2)],v2[c(1,2)])) )
v3 = c(temp1, temp2, temp3)
v1 %*% v3
v2 %*% v3
point3_1 = ooo - .5 * v3
point3_2 = ooo + .5 * v3

### and plot - rotate manually for best visibility
xlim = c (range(scores[,1])[1]-2, range(scores[,1])[2]+1)
ylim = c (range(scores[,2])[1]-2, range(scores[,2])[2]+1)
zlim = c (range(scores[,3])[1]-2, range(scores[,3])[2]+1)

#dfTemp$topCall = as.character(dfTemp$topCall)
#dfTemp$topCall = ifelse (dfTemp$topCall=='callDK','',dfTemp$topCall)
#dfTemp$topCall = as.factor(dfTemp$topCall)

# manually rotate the plot until satisfied, then save the parameters to be used for plotting all 3 lang-s
# paste0( round(as.numeric(par3d()$userMatrix),2), collapse=',')
# zoom<-par3d()$zoom  # suggestion: zoom=0.42
# userMatrix<-par3d()$userMatrix # suggestion: userMatrix = matrix(as.vector(c(0.36,-0.1,-0.92,0,0.42,0.89,0.07,0,0.82,-0.42,0.37,0,0,0,0,1)), ncol=4, nrow=4)
# windowRect<-par3d()$windowRect # suggestion: windowRect = c(11,68,1929,1115)

open3d(zoom = zoom, userMatrix = userMatrix, windowRect=windowRect)
plot3d (scores[,1],scores[,2],scores[,3], box=F, axes=F, type='n', xlab='', ylab='', zlab='', xlim=xlim, ylim=ylim, zlim=zlim) 
text3d (scores[,1],scores[,2],scores[,3], texts=dfTemp$topCall, col=as.numeric(as.factor(dfTemp$col_as_number)), size=1)
segments3d(x=c(point1_1[1], point1_2[1]),
           y=c(point1_1[2], point1_2[2]),
           z=c(point1_1[3], point1_2[3]))
segments3d(x=c(point2_1[1], point2_2[1]),
           y=c(point2_1[2], point2_2[2]),
           z=c(point2_1[3], point2_2[3]))
segments3d(x=c(point3_1[1], point3_2[1]),
           y=c(point3_1[2], point3_2[2]),
           z=c(point3_1[3], point3_2[3]))
text3d(centroids$mds1, centroids$mds2, centroids$mds3, texts=calls_en, cex=2, col='black') 
calls[1]=''
text3d(centroids$mds1+.12, centroids$mds2-.12, centroids$mds3, texts=calls, cex=1.5, col='gray50') # except for English

# snapshot3d  (paste0('~/Downloads/triplets_S_',lang,'.png'))
# movie3d( spin3d(axis=c(0,0,1)), duration=12, dir = '~/Downloads/' )  # make a movie. 12 gives a full 360° circle. 

